<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Api\Controller;

use Psr\Log\LoggerInterface;
use WPDesk\ShopMagic\Api\Normalizer\AutomationNormalizer;
use WPDesk\ShopMagic\Components\Database\Abstraction\PersisterException;
use WPDesk\ShopMagic\Components\Routing\HttpProblemException;
use WPDesk\ShopMagic\Exception\AutomationNotFound;
use WPDesk\ShopMagic\Helper\RestRequestUtil;
use WPDesk\ShopMagic\Workflow\Automation\AutomationNotSaved;
use WPDesk\ShopMagic\Workflow\Automation\AutomationObjectManager;
use WPDesk\ShopMagic\Workflow\Automation\AutomationValueObjectRepository;
use WPDesk\ShopMagic\Workflow\Automation\ValueObject\AutomationValueFactory;

class AutomationController {

	/** @var AutomationValueObjectRepository */
	private $repository;

	/** @var LoggerInterface */
	private $logger;

	public function __construct( AutomationValueObjectRepository $repository, LoggerInterface $logger ) {
		$this->repository = $repository;
		$this->logger     = $logger;
	}

	public function index( \WP_REST_Request $request, AutomationNormalizer $normalizer ): \WP_REST_Response {
		return new \WP_REST_Response(
			$this->repository->find_by( ...$this->parse_params( $request ) )
			                 ->map( \Closure::fromCallable( [ $normalizer, 'normalize' ] ) )
			                 ->to_array()
		);
	}

	private function parse_params( \WP_REST_Request $request ): array {
		$page      = $request->get_param( 'page' );
		$page_size = $request->get_param( 'pageSize' );
		$filters   = $request->get_param( 'filters' );

		$criteria = [];
		if ( isset( $filters['status'] ) && in_array( $filters['status'], [ 'publish', 'draft', 'trash' ], true ) ) {
			$criteria['post_status'] = $filters['status'];
		} else {
			$criteria['post_status'] = 'any';
		}

		if ( isset( $filters['event'] ) ) {
			$criteria['meta_key']   = '_event';
			$criteria['meta_value'] = sanitize_text_field( $filters['event'] );
		}

		if ( isset( $filters['name'] ) ) {
			$criteria['s'] = sanitize_text_field( $filters['name'] );
		}

		return [ $criteria, [], ( ( $page - 1 ) * $page_size ), $page_size ];
	}

	public function show( int $id, AutomationNormalizer $normalizer ): \WP_REST_Response {
		try {
			return new \WP_REST_Response( $normalizer->normalize( $this->repository->find( $id ) ) );
		} catch ( AutomationNotFound $e ) {
			throw new HttpProblemException(
				[
					'title'  => __( 'Automation not found', 'shopmagic-for-woocommerce' ),
					'detail' => sprintf(
						__( 'There is no automation with ID %d', 'shopmagic-for-woocommerce' ),
						$id
					),
				],
				\WP_Http::NOT_FOUND
			);
		}
	}

	public function delete(
		int $id,
		AutomationObjectManager $manager,
		AutomationValueFactory $factory
	): \WP_REST_Response {
		try {
			$automation = $factory->create( $id );
		} catch ( \Exception $e ) {
			throw new HttpProblemException( [
				'title' => 'You want to delete resource which is not automation.',
			], \WP_Http::FORBIDDEN );
		}
		try {
			$success = $manager->delete( $automation );
		} catch ( PersisterException $e ) {
			throw new HttpProblemException( [
				'title'  => esc_html__( 'You are forbidden to delete this automation.', 'shopmagic-for-woocommerce' ),
				'detail' => $e->getMessage(),
			], \WP_Http::FORBIDDEN, $e );
		}

		if ( $success === false ) {
			return new \WP_REST_Response(
				esc_html__( 'Could not delete automation', 'shopmagic-for-woocommerce' ),
				\WP_Http::UNPROCESSABLE_ENTITY
			);
		}

		return new \WP_REST_Response( null, \WP_Http::NO_CONTENT );
	}

	public function create(
		\WP_REST_Request $request,
		\wpdb $wpdb,
		AutomationObjectManager $manager,
		AutomationNormalizer $normalizer
	): \WP_REST_Response {
		$this->logger->debug( 'Starting transaction to create new automation.' );
		try {
			$wpdb->query( 'START TRANSACTION' );
			$automation = $normalizer->denormalize(
				array_merge(
					$request->get_json_params(),
					[ 'id' => null ] // ensure, we don't have dirty ID in request
				)
			);
			$manager->save( $automation );
			$wpdb->query( 'COMMIT' );
			$this->logger->debug( sprintf( 'Automation successfully saved with ID %d', $automation->get_id() ) );
		} catch ( AutomationNotSaved $e ) {
			$wpdb->query( 'ROLLBACK' );
			$this->logger->error( sprintf( 'Automation could not be created. Reason: %s', $e->getMessage() ) );

			throw new HttpProblemException( [
				'title'  => __( 'There are some issues with automation configuration', 'shopmagic-for-woocommerce' ),
				'detail' => $e->getMessage(),
			], \WP_Http::UNPROCESSABLE_ENTITY, $e );
		} catch ( PersisterException $e ) {
			$wpdb->query( 'ROLLBACK' );
			$this->logger->error( sprintf( 'Automation could not be created. Reason: %s', $e->getMessage() ) );

			throw new HttpProblemException( [
				'title'  => __( 'You are not allowed to save this automation.', 'shopmagic-for-woocommerce' ),
				'detail' => $e->getMessage(),
			], \WP_Http::FORBIDDEN, $e );
		} catch ( \Throwable $e ) {
			$wpdb->query( 'ROLLBACK' );
			$this->logger->error( sprintf( 'Automation could not be created. Reason: %s', $e->getMessage() ) );

			throw new HttpProblemException( [
				'title'  => __( 'Critical error during saving automation.', 'shopmagic-for-woocommerce' ),
				'detail' => $e->getMessage(),
			], \WP_Http::INTERNAL_SERVER_ERROR, $e );
		}

		$response = new \WP_REST_Response( $automation->get_id(), 201 );
		$response->header( 'Location', RestRequestUtil::get_url( '/automations/' . $automation->get_id() ) );

		return $response;
	}

	public function update(
		int $id,
		\WP_REST_Request $request,
		\wpdb $wpdb,
		AutomationObjectManager $manager,
		AutomationNormalizer $normalizer
	): \WP_REST_Response {
		$this->logger->debug( sprintf( 'Starting transaction to update automation %d.', $id ) );
		try {
			$wpdb->query( 'START TRANSACTION' );
			$automation = $normalizer->denormalize(
				array_merge(
					$request->get_json_params(),
					[ 'id' => $id ]
				)
			);
			$manager->save( $automation );
			$wpdb->query( 'COMMIT' );
			$this->logger->debug( sprintf( 'Automation successfully saved with ID %d', $id ) );

			return new \WP_REST_Response( $automation->get_id() );
		} catch ( AutomationNotSaved $e ) {
			$wpdb->query( 'ROLLBACK' );
			$this->logger->error( sprintf( 'Automation could not be created. Reason: %s', $e->getMessage() ) );

			throw new HttpProblemException( [
				'title'  => 'Error occurred while saving automation',
				'detail' => $e->getMessage(),
			], \WP_Http::UNPROCESSABLE_ENTITY );
		}
	}

	public function count( \WP_REST_Request $request ): \WP_REST_Response {
		return new \WP_REST_Response( $this->repository->count( ...$this->parse_params( $request ) ) );
	}

}
