<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Api\Normalizer;

use WPDesk\ShopMagic\Workflow\Automation\ValueObject\ActionValueObject;
use WPDesk\ShopMagic\Workflow\Automation\ValueObject\AutomationValueObject;
use WPDesk\ShopMagic\Workflow\Automation\ValueObject\EventValueObject;
use WPDesk\ShopMagic\Workflow\Automation\ValueObject\FilterValueObject;

/**
 * @implements Denormalizer<AutomationValueObject>
 */
class AutomationNormalizer implements Normalizer, Denormalizer {

	/**
	 * @param AutomationValueObject|object $object
	 *
	 * @return array
	 */
	public function normalize( object $object ): array {
		if ( ! $this->supports_normalization( $object ) ) {
			throw InvalidArgumentException::invalid_object( AutomationValueObject::class, $object );
		}

		return [
			'id'          => $object->get_id(),
			'name'        => $object->get_name(),
			'description' => $object->get_description(),
			'status'      => $object->get_status(),
			'event'       => [
				'name'     => $object->get_event()->get_name(),
				'settings' => $object->get_event()->get_settings(),
			],
			'actions'     => array_map(
				function ( ActionValueObject $action ) {
					return [
						'name'     => $action->get_name(),
						'settings' => array_filter( $action->get_settings(), static function ( $item, $key ) {
							if ( $key === 'attachment' && empty( $item ) ) {
								return false;
							}

							return true;
						}, ARRAY_FILTER_USE_BOTH ),
					];
				},
				array_values( $object->get_actions() ?? [] )
			),
			'filters'     => array_map(
				function ( array $filters ) {
					return array_map(
						function ( FilterValueObject $filter ) {
							return array_merge(
								$filter->get_settings(),
								[ 'id' => $filter->get_id() ]
							);
						},
						array_values( $filters )
					);
				},
				array_values( $object->get_filters() )
			),
			'language'    => $object->get_language(),
			'recipe'      => $object->is_from_recipe(),
		];
	}

	public function supports_normalization( object $object ): bool {
		return $object instanceof AutomationValueObject;
	}

	public function denormalize( array $payload ): object {
		$automation = new AutomationValueObject();
		$automation->set_id( isset( $payload['id'] ) ? (int) $payload['id'] : null );
		if ( isset( $payload['status'] ) && $payload['status'] === 'publish' ) {
			$automation->set_status( 'publish' );
		} else {
			$automation->set_status( 'draft' );
		}
		$automation->set_name( sanitize_text_field( $payload['name'] ) );
		$automation->set_event(
			new EventValueObject( $payload['event']['name'], $payload['event']['settings'] )
		);
		$actions = [];
		foreach ( $payload['actions'] ?? [] as $action ) {
			$actions[] = new ActionValueObject( $action['name'], $action['settings'] );
		}
		$automation->set_actions( $actions );

		$filters = [];
		foreach ( array_values( $payload['filters'] ?? [] ) as $k => $filter_group ) {
			$filters[ $k ] = [];
			foreach ( $filter_group as $filter ) {
				$filters[ $k ][] = new FilterValueObject( $filter['id'], $filter );
			}
		}
		$automation->set_filters( $filters );

		if ( isset( $payload['language'] ) ) {
			$automation->set_language( $payload['language'] );
		}

		if ( isset( $payload['recipe'] ) ) {
			$automation->set_from_recipe( (bool) $payload['recipe'] );
		}

		return $automation;
	}

	public function supports_denormalization( array $data ): bool {
		return true;
	}
}
