<?php
declare(strict_types=1);

namespace WPDesk\ShopMagic\Beacon;

use ShopMagicVendor\WPDesk\PluginBuilder\Plugin\Hookable;
use ShopMagicVendor\WPDesk\ShowDecision\ShouldShowStrategy;
use WPDesk\ShopMagic\Admin\Settings\GeneralSettings;
use WPDesk\ShopMagic\Helper\Conditional;
use WPDesk\ShopMagic\Helper\PluginBag;

/**
 * Can display HelpScout Beacon. For more info check https://secure.helpscout.net/settings/beacons/
 */
final class Beacon implements Hookable, Conditional {

	/**
	 * Beacon UUID from HelpScout.
	 *
	 * @var string
	 */
	private $beacon_id;

	/**
	 * When to display beacon.
	 *
	 * @var ShouldShowStrategy
	 */
	private $activation_strategy;

	/**
	 * @var string
	 */
	private $assets_url;

	/**
	 * Beacon constructor.
	 *
	 * @param string             $beacon_id  .
	 * @param ShouldShowStrategy $strategy   When to display beacon.
	 * @param string             $assets_url With ending /
	 */
	public function __construct(
		string $beacon_id,
		ShouldShowStrategy $strategy,
		PluginBag $plugin_bag
	) {
		$this->beacon_id           = $beacon_id;
		$this->activation_strategy = $strategy;
		$this->assets_url          = $plugin_bag->get_assets_url();
	}

	public static function is_needed(): bool {
		return is_admin() && GeneralSettings::get_option( 'disable_beacon', 'no' ) !== 'yes';
	}

	/**
	 * Hooks.
	 */
	public function hooks(): void {
		if ( $this->should_display_beacon() ) {
			add_action(
				'admin_footer',
				function () {
					$this->add_beacon_to_footer();
				}
			);
			add_action(
				'admin_enqueue_scripts',
				function () {
					$this->add_beacon_js();
				}
			);
		}
	}

	/**
	 * Should display beacon?
	 */
	private function should_display_beacon(): bool {
		return $this->activation_strategy->shouldDisplay();
	}

	/**
	 * Display Beacon script.
	 */
	public function add_beacon_to_footer(): void {
		if ( $this->should_display_beacon() ) {
			$beacon_id = $this->beacon_id;
			include __DIR__ . '/templates/html-beacon-script.php';
		}
	}

	public function add_beacon_js(): void {
		if ( $this->should_display_beacon() ) {
			wp_register_script( 'hs-beacon', $this->assets_url . '/js/hs-bc.js', [] );
			wp_enqueue_script( 'hs-beacon' );
		}
	}
}
