<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Components\Database\Collections;

use WPDesk\ShopMagic\Components\Collections\ArrayCollection;
use WPDesk\ShopMagic\Components\Collections\Collection;
use WPDesk\ShopMagic\Components\Database\Abstraction\DAO\ObjectDehydrator;

class LazyObjectCollection implements Collection {

	/** @var array */
	private $raw_data;

	private $object_collection = [];

	private $initialized = false;

	/** @var ObjectDehydrator */
	private $denormalizer;

	public function __construct( array $raw_data, ObjectDehydrator $denormalizer ) {
		$this->raw_data     = $raw_data;
		$this->denormalizer = $denormalizer;
	}

	public function is_empty(): bool {
		return $this->raw_data === [];
	}

	public function getIterator() {
		if ( $this->initialized === false ) {
			$this->initialize();
		}

		return new \ArrayIterator( $this->object_collection );
	}

	private function initialize() {
		foreach ( $this->raw_data as $raw ) {
			$this->object_collection[] = $this->denormalizer->denormalize( $raw );
		}
		$this->initialized = true;
	}

	public function count() {
		return count( $this->raw_data );
	}

	public function map( \Closure $func ): Collection {
		if ( $this->initialized === false ) {
			$this->initialize();
		}

		return new ArrayCollection( array_map( $func,
			$this->object_collection,
			array_keys( $this->object_collection ) ) );
	}


	public function filter( \Closure $func ): Collection {
		if ( $this->initialized === false ) {
			$this->initialize();
		}

		return new ArrayCollection( array_filter( $this->object_collection, $func ) );
	}

	public function reduce( \Closure $func, $initial = null ): Collection {
		if ( $this->initialized === false ) {
			$this->initialize();
		}

		return array_reduce( $this->object_collection, $func, $initial );
	}

	public function to_array(): array {
		if ( $this->initialized === false ) {
			$this->initialize();
		}

		return $this->object_collection;
	}

	public function slice( int $offset, ?int $length = null ): Collection {
		if ( $this->initialized === false ) {
			$this->initialize();
		}

		return new ArrayCollection( array_slice( $this->object_collection, $offset, $length, true ) );
	}

	public function get( $key ) {
		if ( $this->initialized === false ) {
			$this->initialize();
		}

		return $this->object_collection[ $key ] ?? null;
	}

	public function has( $key ): bool {
		if ( $this->initialized === false ) {
			$this->initialize();
		}

		return isset( $this->object_collection[ $key ] );
	}

	public function set( $key, $value ): void {
		if ( $this->initialized === true ) {
			throw new \LogicException('You cant add elements to collection after initialization.');
		}
		$this->object_collection[ $key ] = $value;
	}
}
