<?php
declare(strict_types=1);

namespace WPDesk\ShopMagic\Customer;

use WPDesk\ShopMagic\Exception\CannotCreateGuestException;
use WPDesk\ShopMagic\Guest\Guest;
use WPDesk\ShopMagic\Guest\GuestCustomer;
use WPDesk\ShopMagic\Guest\GuestRepository;

final class CustomerFactory {
	/** @var string */
	public const GUEST_ID_PREFIX = 'g_';

	/** @var GuestRepository */
	private $guest_repository;

	public function __construct( GuestRepository $guest_repository ) {
		$this->guest_repository = $guest_repository;
	}

	/** @param string|int $id */
	public static function id_to_guest_id( $id ): string {
		if ( is_string( $id ) && str_starts_with( self::GUEST_ID_PREFIX, $id ) ) {
			return $id;
		}
		return self::GUEST_ID_PREFIX . $id;
	}

	/**
	 * @param int|string $id For guest id can be string
	 *
	 * @throws CannotCreateGuestException|\Exception
	 */
	public function create_from_id( $id ): \WPDesk\ShopMagic\Customer\Customer {
		if ( ! self::is_customer_guest_id( $id ) ) {
			return $this->create_from_user( new \WP_User( $id ) );
		}

		$numeric_id = self::convert_customer_guest_id_to_number( $id );

		return $this->create_from_guest( $this->guest_repository->get_by_id( $numeric_id ) );
	}

	/** @param string|int $id */
	public static function is_customer_guest_id( $id ): bool {
		return ! is_numeric( $id );
	}

	public function create_from_user( \WP_User $user ): \WPDesk\ShopMagic\Customer\UserAsCustomer {
		return new UserAsCustomer( $user );
	}

	/** @param string|int $id */
	public static function convert_customer_guest_id_to_number( $id ): int {
		return (int) str_replace( self::GUEST_ID_PREFIX, '', (string) $id );
	}

	public function create_from_guest( Guest $guest ): Customer {
		return new GuestCustomer( $guest );
	}

	public function create_from_user_and_order( \WP_User $user, \WC_Order $order ): \WPDesk\ShopMagic\Customer\UserInOrderContextAsCustomer {
		return new UserInOrderContextAsCustomer( $user, $order );
	}

	public function create_from_guest_id( int $id ): Customer {
		return $this->create_from_guest( $this->guest_repository->get_by_id( $id ) );
	}

	public function create_null(): Customer {
		return new NullCustomer();
	}
}
