<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Customer;

use WPDesk\ShopMagic\Components\Collections\ArrayCollection;
use WPDesk\ShopMagic\Components\Collections\Collection;
use WPDesk\ShopMagic\Components\Database\Abstraction\DAO\ObjectRepository;
use WPDesk\ShopMagic\Exception\CannotProvideItemException;
use WPDesk\ShopMagic\Exception\CustomerNotFound;
use WPDesk\ShopMagic\Guest\GuestCustomer;
use WPDesk\ShopMagic\Guest\GuestRepository;

/**
 * @implements ObjectRepository<Customer>
 */
final class CustomerRepository implements ObjectRepository {

	/** @var GuestRepository */
	private $guest_repository;

	public function __construct(
		GuestRepository $guest_repository
	) {
		$this->guest_repository = $guest_repository;
	}

	public function find_all(): Collection {
		return $this->find_by( [] );
	}

	public function find_by( array $criteria, array $order = [], int $offset = 0, ?int $limit = null ): Collection {
		$users  = new \WP_User_Query( [ 'number' => - 1 ] );
		$result = [];
		foreach ( $users->get_results() as $user ) {
			$result[] = new UserAsCustomer( $user );
		}

		return new ArrayCollection( $result );
	}

	public function find_by_email( string $email ): Customer {
		$user = get_user_by( 'email', $email );
		if ( $user instanceof \WP_User ) {
			return new UserAsCustomer( $user );
		}

		try {
			$guest = $this->guest_repository->get_by_email( $email );
		} catch ( CannotProvideItemException $e ) {
			throw new CustomerNotFound( 'Couldn\'t find matching email in sites\' users and guests.' );
		}

		return new GuestCustomer( $guest );
	}

	/** @param \WP_User|numeric $user */
	public function fetch_user( $user ): Customer {
		$wp_user = new \WP_User( $user );
		if ( $wp_user->exists() ) {
			return new UserAsCustomer( $wp_user );
		}

		throw new CustomerNotFound( sprintf( 'Failed to fetch user with ID `%d`.', $user->ID ) );
	}

	/**
	 * @param string|int $id
	 *
	 * @return Customer
	 */
	public function find_by_id( $id ): Customer {
		return $this->find( $id );
	}

	/**
	 * @param string|int $id
	 *
	 * @return Customer
	 */
	public function find( $id ): object {
		if ( CustomerFactory::is_customer_guest_id( $id ) ) {
			try {
				$guest = $this->guest_repository->get_by_id( CustomerFactory::convert_customer_guest_id_to_number( $id ) );
			} catch ( CannotProvideItemException $e ) {
				throw new CustomerNotFound( sprintf( 'Could not retrieve guest %s from database.', $id ), 0, $e );
			}

			return new GuestCustomer( $guest );
		}

		$user = get_user_by( 'id', $id );
		if ( $user instanceof \WP_User ) {
			return new UserAsCustomer( $user );
		}

		throw new CustomerNotFound( sprintf( 'Could not retrieve any user with id `%s`', $id ) );
	}

	public function find_one_by( array $criteria, ?array $order = null ): object {
		$users = new \WP_User_Query( [ 'number' => 1 ] );
		foreach ( $users->get_results() as $user ) {
			return new UserAsCustomer( $user );
		}

		throw new CustomerNotFound( 'Failed to find single customer matching criteria.' );
	}
}
