<?php
declare( strict_types=1 );

namespace WPDesk\ShopMagic\Extensions\Builtin;

use WPDesk\ShopMagic\Extensions\AbstractExtension;
use WPDesk\ShopMagic\Workflow\Placeholder\Builtin\Order;
use WPDesk\ShopMagic\Workflow\Placeholder\Builtin\Product;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Order\OrderCancelled;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Order\OrderCompleted;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Order\OrderFailed;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Order\OrderNew;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Order\OrderNoteAdded;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Order\OrderOnHold;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Order\OrderPaid;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Order\OrderPending;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Order\OrderProcessing;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Order\OrderRefunded;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Order\OrderStatusChanged;
use WPDesk\ShopMagic\Workflow\Event\DeferredStateCheck\OrderStatusDeferredEvent;
use WPDesk\ShopMagic\Workflow\Event\EventMutex;
use WPDesk\ShopMagic\Workflow\Filter\Builtin\Order\OrderItems;
use WPDesk\ShopMagic\Workflow\Filter\Builtin\Order\OrderNoteContent;
use WPDesk\ShopMagic\Workflow\Filter\Builtin\Order\OrderNoteType;
use WPDesk\ShopMagic\Workflow\Placeholder\Helper\DateFormatHelper;
use WPDesk\ShopMagic\Workflow\Placeholder\Helper\PlaceholderUTMBuilder;
use WPDesk\ShopMagic\Workflow\Placeholder\TemplateRendererForPlaceholders;

final class WooCommerceExtension extends AbstractExtension {

	/** @var EventMutex */
	private $event_mutex;

	public function __construct( EventMutex $event_mutex ) {
		$this->event_mutex = $event_mutex;
	}

	public function get_events(): array {
		return [
			'shopmagic_order_new_event'        => new OrderNew( $this->event_mutex ),
			'shopmagic_order_pending_event'    => new OrderStatusDeferredEvent( new OrderPending( $this->event_mutex ), 'pending' ),
			'shopmagic_order_processing_event' => new OrderStatusDeferredEvent( new OrderProcessing(), 'processing' ),
			'shopmagic_order_cancelled_event'  => new OrderStatusDeferredEvent( new OrderCancelled(), 'cancelled' ),
			'shopmagic_order_completed_event'  => new OrderStatusDeferredEvent( new OrderCompleted(), 'completed' ),
			'shopmagic_order_failed_event'     => new OrderStatusDeferredEvent( new OrderFailed(), 'failed' ),
			'shopmagic_order_on_hold_event'    => new OrderStatusDeferredEvent( new OrderOnHold(), 'on-hold' ),
			'shopmagic_order_refunded_event'   => new OrderStatusDeferredEvent( new OrderRefunded(), 'refunded' ),
			'shopmagic_order_status_changed'   => new OrderStatusChanged(),
			'shopmagic_order_status_paid'      => new OrderPaid(),
			'shopmagic_order_note_added'       => new OrderNoteAdded(),
		];
	}

	public function get_filters(): array {
		return [
			'shopmagic_product_purchased_filter' => new OrderItems(), // warning - legacy key.
			'order_note_type'                    => new OrderNoteType(),
			'order_note_content'                 => new OrderNoteContent(),
		];
	}

	public function get_placeholders(): array {
		$utm_builder = new PlaceholderUTMBuilder();
		$renderer    = TemplateRendererForPlaceholders::with_template_dir( 'products_ordered' );
		$date_helper = new DateFormatHelper();

		return [
			new Order\OrderCustomerId(),
			new Order\OrderBillingEmail(),
			new Order\OrderBillingAddress(),
			new Order\OrderBillingAddress2(),
			new Order\OrderBillingCity(),
			new Order\OrderBillingCountry(),
			new Order\OrderBillingFirstName(),
			new Order\OrderBillingFormattedAddress(),
			new Order\OrderBillingLastName(),
			new Order\OrderBillingPostCode(),
			new Order\OrderBillingState(),

			new Order\OrderShippingAddress(),
			new Order\OrderShippingAddress2(),
			new Order\OrderShippingCity(),
			new Order\OrderShippingCountry(),
			new Order\OrderShippingFirstName(),
			new Order\OrderShippingFormattedAddress(),
			new Order\OrderShippingLastName(),
			new Order\OrderShippingPostCode(),
			new Order\OrderShippingState(),
			new Order\OrderShippingMethod(),

			new Order\OrderCustomerNote(),

			new Order\OrderId(),
			new Order\OrderNumber(),
			new Order\OrderTotal(),
			new Order\OrderDateCreated( $date_helper ),
			new Order\OrderDatePaid( $date_helper ),
			new Order\OrderDateCompleted( $date_helper ),
			new Order\OrderAdminUrl(),
			new Order\OrderPaymentUrl( $utm_builder ),
			new Order\OrderPaymentMethod(),
			new Order\OrderProductsSku(),
			new Order\OrderProductsOrdered( $renderer, $utm_builder ),
			new Order\OrderRelatedProducts( $renderer, $utm_builder ),
			new Order\OrderCrossSells( $renderer, $utm_builder ),
			new Order\OrderDetails(),
			new Order\OrderDownloads(),
			new Order\OrderMeta(),

			new Order\OrderNoteContent(),
			new Order\OrderNoteAuthor(),

			new Product\ProductId(),
			new Product\ProductName(),
			new Product\ProductLink( $utm_builder ),
			new Product\ProductMeta(),
		];
	}

}
