<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Guest;

use Psr\Log\LoggerInterface;
use ShopMagicVendor\WPDesk\PluginBuilder\Plugin\Hookable;
use WPDesk\ShopMagic\Database\DatabaseTable;
use WPDesk\ShopMagic\Exception\CannotProvideItemException;
use WPDesk\ShopMagic\Marketing\Subscribers\DAO\SingleListSubscriber;
use WPDesk\ShopMagic\Marketing\Subscribers\DAO\SubscriberObjectRepository;
use WPDesk\ShopMagic\Workflow\Queue\Queue;

/**
 * Converts various data in db extracting/injecting guests.
 */
final class GuestBackgroundConverter implements Hookable {
	use GuestInOrderContextTrait;

	private const HOOK_CONVERT_ORDER_PAGE      = 'shopmagic/core/guest/convert_order_page';
	private const HOOK_CONVERT_OPTIN_PAGE      = 'shopmagic/core/guest/convert_optin_page';
	private const HOOK_CONVERT_OUTCOME_PAGE    = 'shopmagic/core/guest/convert_outcome_page';
	private const CONVERSION_MUTEX_OPTION_NAME = 'shopmagic_guest_conversion';
	private const EXCEPTION                    = 'exception';
	private const GUEST_ID                     = 'guest_id';
	private const LIMIT                        = 10;

	/** @var Queue */
	private $queue;

	/** @var LoggerInterface */
	private $logger;

	/** @var bool */
	private $initialized = false;

	/** @var GuestRepository */
	private $repository;

	/** @var GuestFactory */
	private $factory;

	/** @var SubscriberObjectRepository */
	private $subscribers_repository;

	public function __construct(
		Queue $queue,
		GuestRepository $repository,
		SubscriberObjectRepository $subscribers_repository,
		GuestFactory $factory,
		LoggerInterface $logger
	) {
		$this->queue                  = $queue;
		$this->logger                 = $logger;
		$this->repository             = $repository;
		$this->subscribers_repository = $subscribers_repository;
		$this->factory                = $factory;
	}

	/**
	 * Extract guests from orders and optins if run first time.
	 */
	public function start_guest_extraction_if_needed(): void {
		if ( ! $this->initialized ) {
			$this->logger->warning( self::class . ' must be initialized first.' );

			return;
		}

		$time = microtime( true );
		if ( ! get_option( self::CONVERSION_MUTEX_OPTION_NAME ) ) {
			update_option( self::CONVERSION_MUTEX_OPTION_NAME, $time, true );
			if ( get_option( self::CONVERSION_MUTEX_OPTION_NAME ) === $time ) {
				$this->ensure_guest_tracking_key();
				$this->queue->add(
					self::HOOK_CONVERT_ORDER_PAGE,
					[
						'page' => 0,
					]
				);
			}
		}
	}

	private function ensure_guest_tracking_key(): void {
		global $wpdb;
		$table = DatabaseTable::guest();
		$wpdb->query( sprintf( 'UPDATE %s SET trackig_key = MD5(concat(id, email))', $table ) );
	}

	/**
	 * Initialize the class so it can work properly.
	 */
	public function hooks(): void {
		$this->initialized = true;
		add_action(
			self::HOOK_CONVERT_ORDER_PAGE,
			function ( int $page ) {
				$this->convert_orders_and_queue_next( $page );
			}
		);
		add_action(
			self::HOOK_CONVERT_OPTIN_PAGE,
			function ( int $page ) {
				$this->convert_optins_and_queue_next( $page );
			}
		);
		add_action(
			self::HOOK_CONVERT_OUTCOME_PAGE,
			function ( int $page ) {
				$this->convert_outcomes_and_queue_next( $page );
			}
		);
	}

	/**
	 * Converts page of orders to guests.
	 *
	 * @internal
	 */
	public function convert_orders_and_queue_next( int $page ): void {
		$order = null;
		$guest = null;
		try {
			$orders = wc_get_orders(
				[
					'posts_per_page' => self::LIMIT,
					'page'           => $page,
					'paginate'       => true,
					'order'          => 'ASC',
					'orderby'        => 'date',
				]
			);
			foreach ( $orders->orders as $order ) {
				/** @var \WC_Abstract_Order $order */
				$this->logger->debug( sprintf( 'Guest extraction from order: %s', $order->get_id() ) );
				if ( $this->order_has_guest( $order ) ) {
					try {
						$guest = $this->repository->get_by_email( $order->get_billing_email() );
					} catch ( CannotProvideItemException $e ) {
						$guest = $this->factory->from_order( $order );
						$this->repository->save( $guest );
					}

					$this->touch_order( $order->get_id(), $guest->get_entity()->get_id() );
				}
			}

			if ( ( \is_array( $orders->orders ) || $orders->orders instanceof \Countable ? \count( $orders->orders ) : 0 ) > 0 ) {
				$this->queue->add(
					self::HOOK_CONVERT_ORDER_PAGE,
					[ 'page' => $page + 1 ]
				);
			} else { // after all orders we can convert outcomes and optins.
				$this->queue->add(
					self::HOOK_CONVERT_OUTCOME_PAGE,
					[ 'page' => 0 ]
				);
				$this->queue->add(
					self::HOOK_CONVERT_OPTIN_PAGE,
					[ 'page' => 0 ]
				);
			}
		} catch ( \Throwable $throwable ) {
			$this->logger->error(
				'Error while converting order to guest',
				[
					self::EXCEPTION => $throwable,
					'order_id'      => $order instanceof \WC_Abstract_Order ? $order->get_id() : null,
					self::GUEST_ID  => $guest instanceof Guest ? $guest->get_id() : null,
					'stack'         => $throwable->getTraceAsString(),
				]
			);
		}
	}

	/**
	 * Converts page of orders to guests.
	 *
	 * @internal
	 */
	public function convert_optins_and_queue_next( int $page ): void {
		try {
			$subscribers = $this->subscribers_repository->find_by(
				[ 'active' => '1' ],
				[],
				max( 0, $page - 1 ) * self::LIMIT,
				self::LIMIT
			);
			/** @var SingleListSubscriber $subscriber */
			foreach ( $subscribers as $subscriber ) {
				$this->logger->debug( sprintf( 'Guest extraction from email: %s', $subscriber->get_email() ) );
				if ( ! get_user_by( 'email', $subscriber->get_email() ) ) {
					try {
						$guest = $this->repository->get_by_email( $subscriber->get_email() );
					} catch ( CannotProvideItemException $e ) {
						$guest = $this->factory->from_email( $subscriber->get_email() );
					}

					if ( ! $guest->is_saved() ) {
						$this->repository->save( $guest );
					}
				}
			}

			if ( $subscribers->count() > 0 ) {
				$this->queue->add(
					self::HOOK_CONVERT_OPTIN_PAGE,
					[
						'page' => $page + 1,
					]
				);
			}
		} catch ( \Throwable $throwable ) {
			$this->logger->error(
				'Error while converting optin to guest',
				[
					self::EXCEPTION => $throwable,
					'guest'         => $guest,
				]
			);
		}
	}

	/**
	 * Converts page of outcomes injecting guests.
	 *
	 * @internal
	 */
	public function convert_outcomes_and_queue_next( int $page ): void {
		global $wpdb;
		try {
			$outcomes_table = DatabaseTable::automation_outcome();
			$offset         = max( 0, $page - 1 ) * self::LIMIT;
			$outcomes       = $wpdb->get_results( sprintf( 'SELECT id, customer_email FROM %s WHERE customer_id is NULL LIMIT %s OFFSET %d',
				$outcomes_table,
				self::LIMIT,
				$offset ) );
			foreach ( $outcomes as $outcome_data ) {
				try {
					$guest = $this->repository->get_by_email( $outcome_data->customer_email );
					$wpdb->update( $outcomes_table,
						[ self::GUEST_ID => $guest->get_id() ],
						[ 'id' => $outcome_data->id ] );
				} catch ( CannotProvideItemException $cannotProvideItemException ) {
					$guest = $this->factory->from_email( $outcome_data->customer_email );
					$this->repository->save( $guest );

					$wpdb->update( $outcomes_table,
						[ self::GUEST_ID => $guest->get_id() ],
						[ 'id' => $outcome_data->id ] );
				}
			}

			if ( ( \is_array( $outcomes ) || $outcomes instanceof \Countable ? \count( $outcomes ) : 0 ) > 0 ) {
				$this->queue->add(
					self::HOOK_CONVERT_OUTCOME_PAGE,
					[
						'page' => $page + 1,
					]
				);
			}
		} catch ( \Throwable $throwable ) {
			$this->logger->error(
				'Error while injectings guests to outcomes',
				[
					self::EXCEPTION => $throwable,
				]
			);
		}
	}
}
