<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Integration;

use Psr\Log\LoggerInterface;
use ShopMagicVendor\DI\Container;
use WPDesk\ShopMagic\Admin\Settings\SettingsCollection;
use WPDesk\ShopMagic\Admin\Settings\SettingTab;
use WPDesk\ShopMagic\Components\Routing\RoutesConfigurator;
use WPDesk\ShopMagic\DataSharing\DataProvider;
use WPDesk\ShopMagic\DataSharing\TestProvider;
use WPDesk\ShopMagic\DI\CompositeContainer;
use WPDesk\ShopMagic\Extensions\Extension;
use WPDesk\ShopMagic\Extensions\ExtensionsSet;
use WPDesk\ShopMagic\Helper\PluginBag;
use WPDesk\ShopMagic\Workflow\ActionExecution\ExecutionCreator\ExecutionCreator;
use WPDesk\ShopMagic\Workflow\ActionExecution\ExecutionCreator\ExecutionCreatorContainer;
use WPDesk\ShopMagic\Workflow\Automation\AutomationInitializer;
use WPDesk\ShopMagic\Workflow\Automation\AutomationValidator;

/**
 * Class that grants access to some internal classes and info about ShopMagic to external plugins.
 */
final class ExternalPluginsAccess {

	/** @var string */
	private $version;

	/** @var AutomationInitializer */
	private $automation_factory;

	/** @var ExecutionCreatorContainer */
	private $executor_factory;

	/** @var ExtensionsSet */
	private $extensions_set;

	/** @var Container */
	private $container;

	/** @var SettingsCollection */
	private $settings;

	public function __construct(
		PluginBag $plugin_bag,
		AutomationInitializer $automation_factory,
		ExecutionCreatorContainer $executor_factory,
		ExtensionsSet $extensions_set,
		SettingsCollection $settings,
		Container $container
	) {
		$this->version            = $plugin_bag->get_version();
		$this->automation_factory = $automation_factory;
		$this->executor_factory   = $executor_factory;
		$this->extensions_set     = $extensions_set;
		$this->settings           = $settings;
		$this->container          = $container;
	}

	public function set_validator( AutomationValidator $validator ): void {
		$this->automation_factory->set_validator( $validator );
	}

	public function add_extension( Extension $extension ): void {
		$this->extensions_set->add_extension( $extension );
	}

	public function add_test_provider( DataProvider $provider ): void {
		$this->container->get( TestProvider::class )->add_provider( $provider );
	}

	public function append_setting_tab( SettingTab $tab ): void {
		$this->settings->append_setting_tab( $tab );
	}

	public function get_logger(): LoggerInterface {
		return $this->container->get( 'logger' );
	}

	/**
	 * @todo    refine this comment!
	 * You can extend the logic of how ShopMagic enqueues action to execute.
	 * WARNING: be cautious about adding the strategies because those are executed in order of addition.
	 * I.e. if you add two executors, which both returns `true` in `should_create` method, only the first one will
	 * execute.
	 * @see     QueueExecutionStrategyFactory::create_strategy() for implementation details
	 * @example Delayed Actions add-on
	 */
	public function add_execution_creator( ExecutionCreator $factory ): void {
		$this->executor_factory->add_execution_creator( $factory );
	}

	public function get_version(): string {
		return $this->version;
	}

	public function get_container(): CompositeContainer {
		return new CompositeContainer( [ $this->container ] );
	}

	public function get_routes_configurator(): RoutesConfigurator {
		return $this->container->make( 'routesConfigurator.api' );
	}

}
