<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Marketing\Subscribers\Shortcode;

use ShopMagicVendor\WPDesk\PluginBuilder\Plugin\Hookable;
use WPDesk\ShopMagic\Components\Mailer\Mailer;
use WPDesk\ShopMagic\Customer\Customer;
use WPDesk\ShopMagic\Customer\CustomerFactory;
use WPDesk\ShopMagic\Customer\CustomerRepository;
use WPDesk\ShopMagic\Exception\CustomerNotFound;
use WPDesk\ShopMagic\Guest\GuestFactory;
use WPDesk\ShopMagic\Guest\GuestRepository;
use WPDesk\ShopMagic\Marketing\Segmentation\AudienceListRepository;
use WPDesk\ShopMagic\Marketing\Subscribers\CustomerSubscriberService;
use WPDesk\ShopMagic\Marketing\Subscribers\DAO\SubscriberObjectRepository;

final class FrontendListSubscription implements Hookable {

	/** @var CustomerRepository */
	private $customers_repository;

	/** @var AudienceListRepository */
	private $newsletter_repository;

	/** @var GuestRepository */
	private $guest_repository;

	/** @var CustomerSubscriberService */
	private $subscriber_service;

	/** @var SubscriberObjectRepository */
	private $subscriber_repository;

	public function __construct(
		CustomerSubscriberService $subscriber_service,
		SubscriberObjectRepository $subscriber_repository,
		CustomerRepository $customers_repository,
		GuestRepository $guest_repository,
		AudienceListRepository $newsletter_repository
	) {
		$this->subscriber_service    = $subscriber_service;
		$this->subscriber_repository = $subscriber_repository;
		$this->customers_repository  = $customers_repository;
		$this->guest_repository      = $guest_repository;
		$this->newsletter_repository = $newsletter_repository;
	}

	/** @return void */
	public function hooks() {
		add_action(
			'wp_ajax_' . FrontendForm::ACTION,
			function () {
				$this->try_signup_customer();
			}
		);
		add_action(
			'wp_ajax_nopriv_' . FrontendForm::ACTION,
			function () {
				$this->try_signup_customer();
			}
		);
	}

	/** @return void */
	private function try_signup_customer() {
		check_ajax_referer( FrontendForm::ACTION );

		$name    = isset( $_POST['name'] ) ? sanitize_text_field( wp_unslash( $_POST['name'] ) ) : '';
		$email   = isset( $_POST['email'] ) ? sanitize_email( wp_unslash( $_POST['email'] ) ) : '';
		$list_id = isset( $_POST['list_id'] ) ? absint( wp_unslash( $_POST['list_id'] ) ) : 0;

		if ( empty( $email ) ) {
			wp_send_json_error( esc_html__( 'Please, enter a valid email address!', 'shopmagic-for-woocommerce' ) );
		}

		if ( $this->subscriber_repository->is_subscribed_to_list( $email, $list_id ) ) {
			wp_send_json_error( esc_html__( 'You are already subscribed.', 'shopmagic-for-woocommerce' ) );
		}

		$customer = $this->retrieve_customer( $email, $name );

		if ( isset( $_POST['double_optin'] ) ) {
			( new ConfirmationDispatcher(
				$customer,
				$this->newsletter_repository->find( $list_id ),
				new Mailer()
			) )
				->dispatch_ajax_confirmation();
			die;
		}

		$result = $this->subscriber_service->subscribe( $customer->get_email(), $list_id );

		if ( $result === false ) {
			wp_send_json_error( esc_html__( 'An error occurred during sign up.', 'shopmagic-for-woocommerce' ) );
		}

		wp_send_json_success( esc_html__( 'You have been successfully subscribed!', 'shopmagic-for-woocommerce' ) );
	}

	private function retrieve_customer( string $email, string $name ): Customer {
		try {
			return $this->customers_repository->find_by_email( $email );
		} catch ( CustomerNotFound $e ) {
			$guest_factory = new GuestFactory();
			$guest         = $guest_factory->from_email( $email );
			$guest->set_meta_value( 'first_name', $name );

			$this->guest_repository->save( $guest );

			return ( new CustomerFactory() )
				->create_from_guest( $guest );
		}
	}

}
