<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Marketing\Subscribers\View;

use Exception;
use ShopMagicVendor\WPDesk\View\Renderer\Renderer;
use WPDesk\ShopMagic\Customer\Customer;
use WPDesk\ShopMagic\Customer\CustomerRepository;
use WPDesk\ShopMagic\Marketing\Subscribers\DAO\SubscriberObjectRepository;
use WPDesk\ShopMagic\Marketing\Subscribers\PreferencesRoute;

/**
 * Communication type info for customer. Optins/optouts.
 */
final class AccountPreferences {

	private const CUSTOMER_LOGOUT = 'customer-logout';
	private const SUCCESS         = 'success';
	/** @var SubscriberObjectRepository */
	private $subscribers_repository;
	/** @var Renderer */
	private $renderer;
	/** @var CustomerRepository */
	private $customer_repository;

	public function __construct(
		SubscriberObjectRepository $subscribers_repository,
		Renderer $renderer,
		CustomerRepository $customer_repository
	) {
		$this->subscribers_repository = $subscribers_repository;
		$this->renderer               = $renderer;
		$this->customer_repository = $customer_repository;
	}

	public function hooks(): void {
		if ( ! $this->should_append_page_to_account() ) {
			return;
		}
		add_filter(
			'woocommerce_account_menu_items',
			function ( array $items ): array {
				return $this->new_menu_items( $items );
			}
		);

		add_action(
			'woocommerce_account_' . PreferencesRoute::get_slug() . '_endpoint',
			function (): void {
				$this->nav_menu_content();
			}
		);
	}

	private function should_append_page_to_account(): bool {
		return (bool) apply_filters( 'shopmagic/core/communication_type/account_page_show', true );
	}

	/**
	 * Insert the new endpoint into the My Account menu.
	 *
	 * @param string[] $items
	 *
	 * @return string[]
	 * @internal
	 */
	public function new_menu_items( array $items ): array {
		$logout_item = false;

		if ( isset( $items[ self::CUSTOMER_LOGOUT ] ) ) {
			$logout_item = $items[ self::CUSTOMER_LOGOUT ];
			unset( $items[ self::CUSTOMER_LOGOUT ] );
		}

		$items[ PreferencesRoute::get_slug() ] = $this->get_title();

		if ( $logout_item ) {
			$items[ self::CUSTOMER_LOGOUT ] = $logout_item;
		}

		return $items;
	}

	private function get_title(): string {
		return apply_filters(
			'shopmagic/core/communication_type/account_page_title',
			__( 'Communication', 'shopmagic-for-woocommerce' )
		);
	}

	/**
	 * @internal WooCommerce communication preferences callback.
	 */
	public function nav_menu_content(): void {
		$this->maybe_display_note();
		$this->display_preferences_form( $this->get_customer() );
	}

	private function maybe_display_note(): void {
		echo $this->maybe_get_note();
	}

	private function maybe_get_note(): string {
		if ( isset( $_GET[ self::SUCCESS ] ) ) {
			if ( (int) $_GET[ self::SUCCESS ] === 1 ) {
				$args = [
					'message'     => esc_html__(
						'You have successfully updated your preferences.',
						'shopmagic-for-woocommerce'
					),
					self::SUCCESS => 1,
				];
			} else {
				$args = [
					'message'     => esc_html__(
						'An error occurred during saving your preferences.',
						'shopmagic-for-woocommerce'
					),
					self::SUCCESS => 0,
				];
			}

			return $this->renderer->render( 'preferences_success', $args );
		}

		return '';
	}

	private function display_preferences_form( Customer $customer ): void {
		echo $this->get_preferences_form( $customer );
	}

	private function get_preferences_form( Customer $customer ): string {
		return $this->renderer->render(
			'communication_preferences',
			[
				'email'         => $customer->get_email(),
				'email_display' => $this->get_customer_email_for_display( $customer ),
				'action'        => PreferencesRoute::get_slug(),
				'signed_ups'    => $this->subscribers_repository->find_by(
					[
						'email'  => $customer->get_email(),
						'active' => 1,
					]
				),
			]
		);
	}

	private function get_customer_email_for_display( Customer $customer ): string {
		if ( $customer->is_guest() ) {
			return $this->obfuscate_email( $customer->get_email() );
		}
		if ( ! $this->should_append_page_to_account() ) {
			return $this->obfuscate_email( $customer->get_email() );
		}

		return $customer->get_email();
	}

	private function obfuscate_email( string $email ): string {
		[ $local, $domain ] = explode( '@', $email );

		$obfuscate = static function ( string $string ): string {
			$len = \strlen( $string );

			return implode(
				'',
				array_map(
					static function ( int $key, string $value ) use ( $len ): string {
						return ( $key === 0 || $key === $len - 1 ) ? $value : '*';
					},
					array_keys( str_split( $string ) ),
					array_values( str_split( $string ) )
				)
			);
		};

		return $obfuscate( $local ) . '@' . $domain;
	}

	private function get_customer(): Customer {
		try {
			return $this->customer_repository->find_by_id( $this->determine_customer_id() );
		} catch ( Exception $exception ) {
			$this->display_error();
			die;
		}
	}

	/** @return string ID must be returned as string for compatibility with guest ID. */
	private function determine_customer_id(): string {
		if ( is_user_logged_in() ) {
			return (string) get_current_user_id();
		}

		return isset( $_GET['id'] ) ? sanitize_text_field( wp_unslash( $_GET['id'] ) ) : '';
	}

	private function display_error(): void {
		wp_die(
			esc_html__( 'Sorry, but something is wrong with your request.', 'shopmagic-for-woocommerce' ),
			esc_html__( 'Account preferences', 'shopmagic-for-woocommerce' ),
			400
		);
	}

	public function display_communication_page(
		string $hash = '',
		?bool $success = null
	): \WP_HTTP_Response {
		$hash = sanitize_text_field( wp_unslash( $hash ) );

		if ( empty( $hash ) && isset( $success ) ) {
			$html = $this->renderer->render( 'communication_preferences_wrap_start' );
			$html .= $this->maybe_get_note();
			$html .= $this->renderer->render( 'communication_preferences_wrap_end' );

			return new \WP_HTTP_Response( $html );
		}

		$customer = $this->get_customer();

		if ( ! PreferencesRoute::validate_hash( $customer->get_email(), $hash ) ) {
			$html = $this->renderer->render( 'communication_preferences_wrap_start' );
			$html .= esc_html__( 'Sorry, but something is wrong with your request.', 'shopmagic-for-woocommerce' );
			$html .= $this->renderer->render( 'communication_preferences_wrap_end' );

			return new \WP_HTTP_Response( $html, \WP_Http::FORBIDDEN );
		}

		$html = '';
		if ( $customer->is_guest() || ! $this->should_append_page_to_account() ) {
			$html .= $this->renderer->render( 'communication_preferences_wrap_start' );
		}

		$html .= $this->maybe_get_note();

		$html .= $this->get_preferences_form( $customer );

		if ( $customer->is_guest() || ! $this->should_append_page_to_account() ) {
			$html .= $this->renderer->render( 'communication_preferences_wrap_end' );

			return new \WP_HTTP_Response( $html );
		}

		return new \WP_HTTP_Response( null );
	}

}
