<?php

namespace WPDesk\ShopMagic\Modules\Mulitilingual\Customer;

use WPDesk\ShopMagic\Customer\Customer;
use WPDesk\ShopMagic\Guest\DAO\GuestEntity;
use WPDesk\ShopMagic\Guest\DAO\GuestMetaEntity;
use WPDesk\ShopMagic\Guest\DAO\GuestObjectManager;
use WPDesk\ShopMagic\Modules\Mulitilingual\Language;

/**
 * Stores metadata about Customer language preferences.
 */
final class CustomerLanguagePersistence {

	/** @var Customer */
	private $customer;

	/** @var Language */
	private $language;

	/** @var GuestObjectManager */
	private $guest_manager;

	public function __construct(
		Customer $customer,
		Language $language,
		GuestObjectManager $guest_manager
	) {
		$this->customer         = $customer;
		$this->language         = $language;
		$this->guest_manager = $guest_manager;
	}

	/** @return void */
	public function hooks() {
		add_action( 'shutdown', [ $this, 'persist_language' ], 99 );
	}

	/**
	 * @return void
	 * @internal
	 */
	public function persist_language() {
		if ( $this->customer->is_guest() ) {
			$this->persist_guest_language();
		} else {
			update_user_meta( (int) ( $this->customer->get_id() ),
				Customer::USER_LANGUAGE_META,
				$this->get_preferred_language() );
		}
	}

	/** @return void */
	private function persist_guest_language() {
		/** @var GuestEntity $guest */
		$guest = $this->guest_manager->get_repository()->find_one_by( [ 'email' => $this->customer->get_email()] );
		$guest->add_meta(new GuestMetaEntity(Customer::USER_LANGUAGE_META, $this->get_preferred_language()));
		$this->guest_manager->save( $guest );
	}

	/**
	 * @return string First matched language processed from request header. If none matched, use default site language.
	 */
	private function get_preferred_language(): string {
		$preferred_languages = isset( $_SERVER['HTTP_ACCEPT_LANGUAGE'] ) ? sanitize_text_field( wp_unslash( $_SERVER['HTTP_ACCEPT_LANGUAGE'] ) ) : '';
		$preferred_languages = $this->parse_preferred_browser_languages( $preferred_languages );

		$matching_languages = $this->get_matching_languages( $preferred_languages );

		return $matching_languages[0] ?? $this->language->default_language();
	}

	/** @return string[] */
	private function parse_preferred_browser_languages( string $accepted_languages ): array {
		$result = [];
		foreach ( explode( ',', $accepted_languages ) as $language_qualifier ) {
			$language_qualifier = array_map( 'trim', explode( ';', $language_qualifier ) );
			$result[]           = $language_qualifier[0];
		}

		return $result;
	}

	private function get_matching_languages( array $preferred_languages ): array {
		return array_values( array_intersect( $preferred_languages, $this->language->supported_languages() ) );
	}

}
