<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Modules\Mulitilingual;

use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use WPDesk\ShopMagic\Admin\Settings\ModulesInfoContainer;
use WPDesk\ShopMagic\DI\ContainerAwareTrait;
use WPDesk\ShopMagic\Exception\CannotProvideCustomerException;
use WPDesk\ShopMagic\Frontend\Interceptor\CurrentCustomer;
use WPDesk\ShopMagic\Marketing\Segmentation\AudienceListFactory;
use WPDesk\ShopMagic\Marketing\Segmentation\AudienceListRepository;
use WPDesk\ShopMagic\Modules\Module;
use WPDesk\ShopMagic\Modules\Mulitilingual\Customer\CustomerLanguagePersistence;
use WPDesk\ShopMagic\Modules\Mulitilingual\Integration\BareHandler;
use WPDesk\ShopMagic\Modules\Mulitilingual\Marketing\MultilingualAudienceListRepository;
use WPDesk\ShopMagic\Modules\Mulitilingual\Validator\CustomerLanguageValidator;
use WPDesk\ShopMagic\Workflow\Automation\AutomationInitializer;
use WPDesk\ShopMagic\Workflow\Automation\AutomationValueObjectRepository;

final class MultilingualModule implements Module, LoggerAwareInterface {
	use ContainerAwareTrait;
	use LoggerAwareTrait;

	/** @var AutomationValueObjectRepository */
	private $automation_repository;

	/** @var AutomationInitializer */
	private $automation_initializer;

	public function __construct(
		AutomationValueObjectRepository $automation_repository,
		AutomationInitializer $automation_initializer
	) {
		$this->automation_repository  = $automation_repository;
		$this->automation_initializer = $automation_initializer;
	}

	public function initialize(): void {
		$language_handler = new BareHandler( $this->automation_repository );
		$this->automation_initializer->set_validator(
			$this->automation_initializer
				->get_validator()
				->add_validator(
					new CustomerLanguageValidator(
						$language_handler
					)
				)
		);

		if ( ! is_admin() ) {
			try {
				$this->container
					->get( CustomerLanguagePersistence::class )
					->hooks();

				$this->container->set(
					AudienceListRepository::class,
					new MultilingualAudienceListRepository(
						$this->container->get( AudienceListFactory::class ),
						$this->container->get( CurrentCustomer::class )->get_customer()
					)
				);
			} catch ( CannotProvideCustomerException $e ) {
				$this->logger->warning( 'Unable to detect current customer to persist language information.' );
			}
		}
	}

	public function is_enabled(): bool {
		return $this->container->get( ModulesInfoContainer::class )->get( $this->get_name() );
	}

	public function get_name(): string {
		return 'multilingual-module';
	}

}
