<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic;

use Psr\Container\ContainerInterface;
use ShopMagicVendor\DI\ContainerBuilder;
use ShopMagicVendor\WPDesk\Migrations\Migrator;
use ShopMagicVendor\WPDesk\PluginBuilder\Plugin\AbstractPlugin;
use ShopMagicVendor\WPDesk\PluginBuilder\Plugin\Activateable;
use ShopMagicVendor\WPDesk\PluginBuilder\Plugin\Deactivateable;
use ShopMagicVendor\WPDesk\PluginBuilder\Plugin\HookableCollection;
use ShopMagicVendor\WPDesk\PluginBuilder\Plugin\HookableParent;
use ShopMagicVendor\WPDesk\ShowDecision\PostTypeStrategy;
use ShopMagicVendor\WPDesk_Plugin_Info;
use WPDesk\ShopMagic\Admin\Admin;
use WPDesk\ShopMagic\Admin\RateNotice\RateNotices;
use WPDesk\ShopMagic\Admin\RateNotice\TwoWeeksNotice;
use WPDesk\ShopMagic\Admin\Settings\GeneralSettings;
use WPDesk\ShopMagic\Admin\Welcome\Welcome;
use WPDesk\ShopMagic\Components\Routing\RestRoutesRegistry;
use WPDesk\ShopMagic\Components\Routing\WpRoutesRegistry;
use WPDesk\ShopMagic\Extensions\Builtin\CoreExtension;
use WPDesk\ShopMagic\Extensions\Builtin\FlexibleCheckoutFieldsExtension;
use WPDesk\ShopMagic\Extensions\Builtin\FlexibleShippingExtension;
use WPDesk\ShopMagic\Extensions\Builtin\PaidPromotionExtension;
use WPDesk\ShopMagic\Extensions\Builtin\StagingExtension;
use WPDesk\ShopMagic\Extensions\Builtin\WooCommerceExtension;
use WPDesk\ShopMagic\Extensions\ExtensionsSet;
use WPDesk\ShopMagic\Guest\GuestBackgroundConverter;
use WPDesk\ShopMagic\Helper\Conditional;
use WPDesk\ShopMagic\Helper\PluginBag;
use WPDesk\ShopMagic\Helper\TemplateResolver;
use WPDesk\ShopMagic\Helper\WordPressPluggableHelper;
use WPDesk\ShopMagic\Integration\ExternalPluginsAccess;
use WPDesk\ShopMagic\Modules\ModulesSet;
use WPDesk\ShopMagic\Modules\Mulitilingual\MultilingualModule;
use WPDesk\ShopMagic\Workflow\Action\Action;
use WPDesk\ShopMagic\Workflow\Automation\AutomationInitializer;
use WPDesk\ShopMagic\Workflow\Automation\AutomationPostType;
use WPDesk\ShopMagic\Workflow\Event\Builtin\Order\OrderPending;
use WPDesk\ShopMagic\Workflow\Event\Event;
use WPDesk\ShopMagic\Workflow\Event\EventMutex;
use WPDesk\ShopMagic\Workflow\Filter\ComparisonType\ComparisonType;
use WPDesk\ShopMagic\Workflow\Filter\Filter;
use WPDesk\ShopMagic\Workflow\Placeholder\Placeholder;
use WPDesk\ShopMagic\Workflow\Queue\Queue;

/**
 * Main plugin class. The most important flow decisions are made here.
 */
final class Plugin extends AbstractPlugin implements HookableCollection, Activateable, Deactivateable {
	use HookableParent;

	private const ADDONS = [
		'shopmagic-advanced-filters/shopmagic-advanced-filters.php',
		'shopmagic-customer-coupons/shopmagic-customer-coupons.php',
		'shopmagic-delayed-actions/shopmagic-delayed-actions.php',
		'shopmagic-for-gravity-forms/shopmagic-for-gravity-forms.php',
		'shopmagic-manual-actions/shopmagic-manual-actions.php',
		'shopmagic-reviews/shopmagic-reviews.php',
		'shopmagic-slack/shopmagic-slack.php',
		'shopmagic-woocommerce-bookings/shopmagic-woocommerce-bookings.php',
		'shopmagic-woocommerce-memberships/shopmagic-woocommerce-memberships.php',
		'shopmagic-woocommerce-subscriptions/shopmagic-woocommerce-subscriptions.php',
	];
	/** @var WPDesk_Plugin_Info */
	protected $plugin_info;
	/** @var ContainerInterface */
	private $container;

	public function __construct( WPDesk_Plugin_Info $plugin_info ) {
		/** @noinspection PhpParamsInspection */
		parent::__construct( $plugin_info );

		$this->set_class_aliases();

		TemplateResolver::set_root_path( $plugin_info->get_plugin_dir() );
	}

	private function set_class_aliases(): void {
		class_alias( self::class, 'ShopMagic' );
		class_alias( Placeholder::class, \WPDesk\ShopMagic\Placeholder\BasicPlaceholder::class );
		class_alias( Event::class, \WPDesk\ShopMagic\Event\BasicEvent::class );
		class_alias( Action::class, \WPDesk\ShopMagic\Action\BasicAction::class );
		class_alias( Filter::class, \WPDesk\ShopMagic\Filter\BasicFilter::class );
		class_alias( ComparisonType::class, WPDesk\ShopMagic\Filter\ComparisionType\ComparisionType::class );
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\CheckboxField::class,
			\WPDesk\ShopMagic\FormField\Field\CheckboxField::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\Header::class,
			\WPDesk\ShopMagic\FormField\Field\Header::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\HiddenField::class,
			\WPDesk\ShopMagic\FormField\Field\HiddenField::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\InputTextField::class,
			\WPDesk\ShopMagic\FormField\Field\InputTextField::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\MultipleInputTextField::class,
			\WPDesk\ShopMagic\FormField\Field\MultipleInputTextField::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\ImageInputField::class,
			\WPDesk\ShopMagic\FormField\Field\ImageInputField::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\NoOnceField::class,
			\WPDesk\ShopMagic\FormField\Field\NoOnceField::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\NoValueField::class,
			\WPDesk\ShopMagic\FormField\Field\NoValueField::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\Paragraph::class,
			\WPDesk\ShopMagic\FormField\Field\Paragraph::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\ProductSelect::class,
			\WPDesk\ShopMagic\FormField\Field\ProductSelect::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\RadioField::class,
			\WPDesk\ShopMagic\FormField\Field\RadioField::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\SelectField::class,
			\WPDesk\ShopMagic\FormField\Field\SelectField::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\SubmitField::class,
			\WPDesk\ShopMagic\FormField\Field\SubmitField::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\TextAreaField::class,
			\WPDesk\ShopMagic\FormField\Field\TextAreaField::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\WooSelect::class,
			\WPDesk\ShopMagic\FormField\Field\WooSelect::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\WyswigField::class,
			\WPDesk\ShopMagic\FormField\Field\WyswigField::class
		);

		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\NoValueField::class,
			\WPDesk\ShopMagic\FormField\NoValueField::class
		);
		class_alias(
			\ShopMagicVendor\WPDesk\Forms\Field\BasicField::class,
			\WPDesk\ShopMagic\FormField\BasicField::class
		);
	}

	public function init(): void {
		$builder = new ContainerBuilder();
		$builder->addDefinitions(
			$this->plugin_info->get_plugin_dir() . '/config/services.inc.php'
		);
		if (!getenv('DEVELOPMENT')) {
			$builder->enableCompilation( __DIR__ . '/../cache' );
		}

		$this->container = $builder->build();

		$plugin_info_bag = $this->container->get( PluginBag::class );
		$plugin_info_bag->set_plugin_info( $this->plugin_info );
		$plugin_info_bag->set_debug( $this->is_debug_mode() );
		$plugin_info_bag->set_pro( $this->is_pro_active() );

		$this->container->get( Migrator::class )->migrate();

		parent::init();
	}

	/**
	 * Check if any of ShopMagic PRO add-ons is active
	 */
	private function is_pro_active(): bool {
		foreach ( self::ADDONS as $addon ) {
			if ( WordPressPluggableHelper::is_plugin_active( $addon ) ) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Returns true when debug mode is on.
	 */
	private function is_debug_mode(): bool {
		$enable_log = GeneralSettings::get_option( 'sm_enable_logs' );
		if ( $enable_log === '1' || $enable_log === true ) {
			return true;
		}
		$helper_options = get_option( 'wpdesk_helper_options', [] );

		return isset( $helper_options['debug_log'] ) && '1' === $helper_options['debug_log'];
	}

	/**
	 * Integrate with WordPress and with other plugins using action/filter system.
	 */
	public function hooks(): void {
		parent::hooks();

		// $pro_is_active = $this->is_pro_active();
		// require_once __DIR__ . '/Admin/Welcome/Popups.php';

		add_action(
			'plugins_loaded',
			function (): void {
				OrderPending::initialize_pending_on_created_check( $this->container->get( Queue::class ) );

				foreach ( $this->container->get( 'hookable.plugins_loaded' ) as $item ) {
					$item->hooks();
				}

				$this->container->get( GuestBackgroundConverter::class )
				                ->start_guest_extraction_if_needed();
			}
		);

		add_action(
			'rest_api_init',
			function () {
				$routes       = require $this->plugin_info->get_plugin_dir() . '/config/routes/api.php';
				$configurator = $this->container->get( 'routesConfigurator.api' );
				$registry     = $this->container->get( RestRoutesRegistry::class );
				$routes( $configurator );
				do_action( 'shopmagic/core/rest/init' );
				$registry->hooks();
			}
		);

		add_action(
			'init',
			function (): void {
				$extensions_set = $this->container->get( ExtensionsSet::class );
				$modules_set    = $this->container->get( ModulesSet::class );
				$this->register_extensions( $extensions_set );
				$this->register_modules( $modules_set );

				/**
				 * If you want to write an integration with ShopMagic you should use this action.
				 * This action is executed when the ShopMagic core is ready to be used and provides a ExternalPluginsAccess object to facilitate integration.
				 * Please make sure that your integration is checking $plugin_access->get_version() to ensure that your integration is compatible with current ShopMagic version.
				 * Remember that we use semantic versioning so can be sure that every time we make a breaking change we also increase a major version of the plugin.
				 *
				 * @param ExternalPluginsAccess $plugin_access Object with various tools that can be used for integration.
				 */
				do_action(
					'shopmagic/core/initialized/v2',
					$this->container->get( ExternalPluginsAccess::class )
				);

				$extensions_set->init_extensions();
				$modules_set->init_modules();

				foreach ( $this->container->get( 'hookable.init' ) as $item ) {
					if ( $item instanceof Conditional && ! $item::is_needed() ) {
						continue;
					}
					$item->hooks();
				}

				$this->register_public_routes();

				$automations = $this->container->get( AutomationInitializer::class )
				                               ->initialize_active_automations();

				do_action(
					'shopmagic/core/after_initialization',
					$this->container->get( ExtensionsSet::class )
				);

				if ( is_admin() ) {
					( new RateNotices(
						[
							new TwoWeeksNotice(
								$this->plugin_url . '/assets',
								$automations,
								new PostTypeStrategy( AutomationPostType::TYPE )
							),
						]
					) )->hooks();
				}
			},
			9
		);

		if ( is_admin() ) {
			$this->container->get( Admin::class )->hooks();
		}

		$this->hooks_on_hookable_objects();
	}

	private function register_extensions( ExtensionsSet $extensions ): void {
		$extensions->add_extension( $this->container->get( CoreExtension::class ) );

		if ( WordPressPluggableHelper::is_plugin_active( 'woocommerce/woocommerce.php' ) ) {
			$extensions->add_extension( new WooCommerceExtension( new EventMutex() ) );
		}

		$extensions->add_extension( new PaidPromotionExtension( $this->is_pro_active() ) );
		$extensions->add_extension( new FlexibleShippingExtension() );

		add_action(
			'flexible_checkout_fields/init',
			function ( $integrator ) use ( $extensions ): void {
				$extensions->add_extension(
					new FlexibleCheckoutFieldsExtension(
						$integrator,
						$this->container->get( 'logger' )
					)
				);
			}
		);

		$extensions->add_extension( new StagingExtension() );
	}

	private function register_modules( ModulesSet $modules ) {
		$modules->add_module( $this->container->get( MultilingualModule::class ) );
	}

	private function register_public_routes() {
		$routes   = require $this->plugin_info->get_plugin_dir() . '/config/routes/public.php';
		$registry = $this->container->get( WpRoutesRegistry::class );
		$routes( $this->container->get( 'routesConfigurator.public' ) );
		$registry->hooks();
	}

	/**
	 * Quick links on plugins page.
	 *
	 * @param string[] $links .
	 *
	 * @return string[]
	 * @internal
	 */
	public function links_filter( $links ): array {
		$plugin_links = [];

		$plugin_links[] = '<a href="' . admin_url( 'admin.php?page=shopmagic-admin#/settings' ) . '">' . __(
				'Settings',
				'shopmagic-for-woocommerce'
			) . '</a>';
		$plugin_links[] = '<a href="https://shopmagic.app/docs/" target="_blank">' . __(
				'Docs',
				'shopmagic-for-woocommerce'
			) . '</a>';
		$plugin_links[] = '<a href="https://wordpress.org/support/plugin/shopmagic-for-woocommerce/" target="_blank">' . __(
				'Support',
				'shopmagic-for-woocommerce'
			) . '</a>';

		if ( ! $this->is_pro_active() ) {
			$plugin_links[] = '<a href="https://shopmagic.app/pricing/?utm_source=user-site&utm_medium=quick-link&utm_campaign=shopmagic-upgrade" target="_blank" style="color:#d64e07;font-weight:bold;">' . __(
					'Buy PRO',
					'shopmagic-for-woocommerce'
				) . '</a>';
		}

		return array_merge( $plugin_links, $links );
	}

	public function activate(): void {
		( new Welcome() )->welcome_activate();
	}

	public function deactivate(): void {
		( new Welcome() )->welcome_deactivate();
	}
}
