<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Recipe;

use WPDesk\ShopMagic\Extensions\ExtensionsSet;
use WPDesk\ShopMagic\Workflow\Action\ActionList;
use WPDesk\ShopMagic\Workflow\Automation\AutomationPersistence;
use WPDesk\ShopMagic\Workflow\Automation\AutomationPostType;
use WPDesk\ShopMagic\Workflow\Event\EventsList;
use WPDesk\ShopMagic\Workflow\Filter\FiltersList;

final class Recipe {
	/**
	 * @var string
	 */
	public const RECIPE_META_NAME = 'shopmagic_source_recipe';
	/**
	 * @var string
	 */
	private const EVENT = 'event';
	/** @var array<string, mixed[]>|array<string, string> $recipe_data */
	private $recipe_data = [];
	/** @var string */
	private $id;
	/** @var bool */
	private $can_use;

	/**
	 * @param array{name:string,description:string,actions:array,event:array,filters:array} $recipe_data
	 * @param string                                                                        $id
	 * @param \WPDesk\ShopMagic\Extensions\ExtensionsSet                                    $extensions_set
	 */
	public function __construct( array $recipe_data, string $id, ExtensionsSet $extensions_set ) {
		$this->recipe_data = $recipe_data;
		$this->id          = $id;
		$this->can_use     = $this->can_use_event( $extensions_set->get_events() ) &&
		                     $this->can_use_actions( $extensions_set->get_actions() ) &&
		                     $this->can_use_filters( $extensions_set->get_filters() );
	}

	private function can_use_event( EventsList $events ): bool {
		return isset( $events[ $this->recipe_data[ self::EVENT ]['slug'] ] );
	}

	private function can_use_actions( ActionList $actions ): bool {
		foreach ( $this->recipe_data['actions'] as $action ) {
			if ( ! isset( $actions[ $action['_action'] ] ) ) {
				return false;
			}
		}

		return true;
	}

	private function can_use_filters( FiltersList $filters ): bool {
		foreach ( $this->recipe_data['filters'] as $or ) {
			foreach ( $or as $filter ) {
				if ( ! isset( $filters[ $filter['filter_slug'] ] ) ) {
					return false;
				}
			}
		}

		return true;
	}

	public function get_id(): string {
		return $this->id;
	}

	public function get_name(): string {
		return $this->recipe_data['name'];
	}

	public function get_description(): string {
		return $this->recipe_data['description'];
	}

	public function can_use(): bool {
		return $this->can_use;
	}

	public function import(): int {
		$id = wp_insert_post(
			[
				'post_title' => $this->recipe_data['name'],
				'post_type'  => AutomationPostType::TYPE,
			]
		);

		if ( ! is_wp_error( $id ) ) {
			$persistence = new AutomationPersistence( $id );
			$persistence->set_event_data( $this->recipe_data[ self::EVENT ]['data'],
				$this->recipe_data[ self::EVENT ]['slug'] );
			$persistence->set_actions_data( $this->recipe_data['actions'] );
			$persistence->set_filters_data( $this->recipe_data['filters'] );
			$persistence->set(self::RECIPE_META_NAME, $this->id);

			return (int) $id;
		}

		return 0;
	}
}
