<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Action\Builtin\OptinList;

use Psr\Container\NotFoundExceptionInterface;
use ShopMagicVendor\WPDesk\Forms\Field\CheckboxField;
use WPDesk\ShopMagic\Components\Mailer\Mailer;
use WPDesk\ShopMagic\Exception\CannotModifyList;
use WPDesk\ShopMagic\Exception\CannotProvideItemException;
use WPDesk\ShopMagic\Marketing\Segmentation\AudienceListRepository;
use WPDesk\ShopMagic\Marketing\Subscribers\DAO\SingleListSubscriber;
use WPDesk\ShopMagic\Marketing\Subscribers\DAO\SingleListSubscriberHydrator;
use WPDesk\ShopMagic\Marketing\Subscribers\DAO\SubscriptionManager;
use WPDesk\ShopMagic\Marketing\Subscribers\Shortcode\ConfirmationDispatcher;

final class AddToListAction extends AbstractListAction {

	/** @var AudienceListRepository */
	private $marketing_list_repository;

	public function __construct(
		AudienceListRepository $marketing_list_manager,
		SubscriptionManager $manager,
		SingleListSubscriberHydrator $factory
	) {
		parent::__construct( $manager, $factory );
		$this->marketing_list_repository = $marketing_list_manager;
	}

	public function get_name(): string {
		return esc_html__( 'Add E-mail to List', 'shopmagic-for-woocommerce' );
	}

	public function get_description(): string {
		return esc_html__('Add customer email to selected marketing list. Also features double opt-in option.', 'shopmagic-for-woocommerce');
	}

	/**
	 * @return \ShopMagicVendor\WPDesk\Forms\Field[]
	 */
	public function get_fields(): array {
		return array_merge(
			parent::get_fields(),
			[
				( new CheckboxField() )
					->set_name( 'doubleoptin' )
					->set_label( esc_html__( 'Double opt-in', 'shopmagic-for-woocommerce' ) ),
			]
		);
	}

	protected function do_list_action( string $email, int $list_id, string $list_name ): bool {
		try {
			/** @var SingleListSubscriber $customer_opt */
			$customer_opt = $this->repository->get_subscribed_to_list( $email, $list_id );
		} catch ( CannotProvideItemException $cannotProvideItemException ) {
			$customer_opt = $this->hydrator->create_for_email_and_list( $email, $list_id );
		}

		if ( $customer_opt->get_id() > 0 && $customer_opt->is_active() ) {
			throw new CannotModifyList(
				sprintf(
				// translators: %1$s Customer email, %2$s Marketing List name.
					esc_html__( 'Customer email %1$s is already subscribed to this list: %2$s.',
						'shopmagic-for-woocommerce' ),
					$email,
					$list_name
				)
			);
		}

		try {
			if ( $this->fields_data->has( 'doubleoptin' ) && $this->fields_data->get( 'doubleoptin' ) === CheckboxField::VALUE_TRUE && $this->is_customer_provided() ) {
				$target_list = $this->marketing_list_repository->find( $list_id );

				return ( new ConfirmationDispatcher( $this->get_customer(), $target_list, new Mailer() ) )
					->dispatch_confirmation_email();
			}
		} catch ( NotFoundExceptionInterface $e ) {
			// Backward compatibility. Process form as if the element wasn't there.
		}

		$customer_opt->set_active( true );
		$this->manager->save( $customer_opt );

		if ( $this->logger !== null ) {
			$this->logger->info(
				sprintf(
				// translators: %1$s Customer email, %2$s Marketing List name.
					esc_html__( 'Customer email %1$s successfully added to list: %2$s.', 'shopmagic-for-woocommerce' ),
					$email,
					$list_name
				)
			);
		}

		return true;
	}
}
