<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\ActionExecution;

use ShopMagicVendor\WPDesk\PluginBuilder\Plugin\Hookable;
use WPDesk\ShopMagic\Workflow\Automation\AutomationInitializer;
use WPDesk\ShopMagic\Workflow\Event\DataLayer;


/**
 * Runs specific action from ActionScheduler's queue hook.
 */
final class QueueActionRunner implements Hookable {
	public const ACTION_EXECUTE_FROM_QUEUE = 'shopmagic/core/queue/execute';
	public const QUEUE_GROUP               = 'shopmagic-automation';

	/** @var AutomationInitializer */
	private $automation_factory;

	/** @var ExecuteNow */
	private $executor;

	public function __construct( AutomationInitializer $factory, ExecuteNow $executor ) {
		$this->automation_factory = $factory;
		$this->executor           = $executor;
	}

	public function hooks(): void {
		add_action(
			self::ACTION_EXECUTE_FROM_QUEUE,
			function (
				array $automation_serialized,
				array $event_serialized,
				array $action_serialized,
				int $action_index,
				?string $unique_id = null
			): void {
				$this->run_action(
					$automation_serialized,
					$event_serialized,
					$action_serialized,
					$action_index,
					$unique_id
				);
			},
			10,
			5
		);
	}

	/**
	 * @param array{id: int} $automation_serialized
	 * @param array          $event_serialized
	 * @param array          $action_serialized
	 * @param int            $action_index
	 * @param string|null    $unique_id
	 */
	private function run_action(
		array $automation_serialized,
		array $event_serialized,
		array $action_serialized,
		int $action_index,
		?string $unique_id = null
	): void {
		if ( $unique_id === null ) {
			$unique_id = uniqid( 'fallback_', true );
		}

		$run = function () use ( $automation_serialized, $event_serialized, $action_index, $unique_id ): void {
			$automation = $this->automation_factory->create_automation( $automation_serialized['id'] );
			/** @note Exit function if automation no longer exists. */
			if ( $automation->get_id() === 0 ) {
				return;
			}
			$event      = $automation->get_event();
			$event->set_runner( $this->automation_factory->create_runner( $automation ) );
			$event->set_data_layer( new DataLayer( [ \get_class( $automation ) => $automation ] ) );
			$event->set_from_json( $event_serialized );

			$this->executor->execute(
				$automation,
				$event,
				$automation->get_action( $action_index ),
				$action_index,
				$unique_id
			);
		};

		if ( \defined( 'DOING_CRON' ) && DOING_CRON ) {
			$run();
		} elseif ( did_action( 'wp_loaded' ) ) {
			$run();
		} else {
			add_action( 'wp_loaded', $run );
		}
	}

}
