<?php
declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Automation;

use Psr\Log\LoggerInterface;
use WPDesk\ShopMagic\DataSharing\FieldValuesBag;
use WPDesk\ShopMagic\Extensions\ExtensionsSet;
use WPDesk\ShopMagic\Workflow\Action\Action;
use WPDesk\ShopMagic\Workflow\Event\Event;
use WPDesk\ShopMagic\Workflow\Event\NullEvent;
use WPDesk\ShopMagic\Workflow\Filter\FilterGroupLogic;
use WPDesk\ShopMagic\Workflow\Filter\NullFilter;

class AutomationFactory {

	/** @var ExtensionsSet */
	private $extensions_set;

	/** @var LoggerInterface */
	private $logger;

	public function __construct( ExtensionsSet $extensions_set, LoggerInterface $logger ) {
		$this->extensions_set = $extensions_set;
		$this->logger = $logger;
	}

	public function create( int $id ): Automation {
		try {
			$persistence = new AutomationPersistence( $id );

			return new Automation(
				$id,
				$this->get_event_for_automation( $persistence ),
				$this->get_filters_for_automation( $persistence ),
				$this->get_actions_for_automation( $persistence )
			);
		} catch ( \Throwable $e ) {
			return new Automation( 0, new NullEvent(), new NullFilter(), [] );
		}
	}

	private function get_event_for_automation( PersistentAutomation $persistence ): Event {
		$event = $this->extensions_set->get_event( $persistence->get_event_slug() );
		$event->update_fields_data( new FieldValuesBag( $persistence->get_event_data() ) );

		return $event;
	}

	private function get_filters_for_automation( PersistentAutomation $persistence ): FilterGroupLogic {
		$filters = [];
		foreach ( $persistence->get_filters_data() as $or_group_index => $and_group ) {
			$filters[ $or_group_index ] = [];
			foreach ( $and_group as $and_group_index => $data ) {
				if ( ! empty( $data[ AutomationPersistence::FILTER_DATA_KEY ] ) ) {
					$filter = $this->extensions_set->get_filter( $data[ AutomationPersistence::FILTER_SLUG_KEY ] );
					$filter->update_fields_data( new FieldValuesBag( $data[ AutomationPersistence::FILTER_DATA_KEY ] ) );

					$filters[ $or_group_index ][ $and_group_index ] = $filter;
				}
			}
		}

		$filter_group_logic = new FilterGroupLogic( $filters );
		$filter_group_logic->setLogger( $this->logger );

		return $filter_group_logic;
	}

	/** @return \WPDesk\ShopMagic\Workflow\Action\Action[] */
	private function get_actions_for_automation( PersistentAutomation $persistence ): array {
		return array_map(
			function ( array $action_data ): Action {
				$action = $this->extensions_set->get_action( $action_data['_action'] ?? null );
				$action->update_fields_data( new FieldValuesBag( $action_data ) );

				return $action;
			},
			$persistence->get_actions_data()
		);
	}
}
