<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Automation;

use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use WPDesk\ShopMagic\Workflow\ActionExecution\ExecutionCreator\ExecutionCreator;
use WPDesk\ShopMagic\Workflow\Automation\Validator\FiltersValidator;
use WPDesk\ShopMagic\Workflow\Automation\Validator\FullyConfiguredValidator;
use WPDesk\ShopMagic\Workflow\Event\DataLayer;
use WPDesk\ShopMagic\Workflow\Outcome\OutcomeManager;
use WPDesk\ShopMagic\Workflow\Outcome\OutcomeSavingState;

/**
 * Can create automation from given id.
 */
final class AutomationInitializer implements LoggerAwareInterface {
	use LoggerAwareTrait;

	/** @var Automation[] */
	private $automations = [];

	/** @var ExecutionCreator */
	private $execution_factory;

	/** @var AutomationValidator|null */
	private $validator;

	/** @var OutcomeManager */
	private $outcome_saver;

	/** @var AutomationFactory */
	private $automation_factory;

	public function __construct(
		AutomationFactory $automation_factory,
		ExecutionCreator $execution_factory,
		OutcomeSavingState $outcome_saver
	) {
		$this->automation_factory = $automation_factory;
		$this->execution_factory  = $execution_factory;
		$this->outcome_saver      = $outcome_saver;
	}

	/** @return Automation[] */
	public function initialize_active_automations(): array {
		$WP_posts = $this->get_active_automations();
		foreach ( $WP_posts as $automation_post ) {
			$this->initialize_automation( (int) $automation_post->ID );
		}

		return $this->automations;
	}

	/** @return \WP_Post[] */
	private function get_active_automations(): array {
		return get_posts(
			[
				'post_type'              => 'shopmagic_automation',
				'post_status'            => 'publish',
				'posts_per_page'         => - 1,
				'no_found_rows'          => true,
				'update_post_term_cache' => false,
				'update_post_meta_cache' => true,
			]
		);
	}

	public function initialize_automation( int $automation_id ): Automation {
		$automation = $this->create_automation( $automation_id );
		$runner     = $this->create_runner( $automation );
		$event      = $automation->get_event();
		$event->set_runner( $runner );
		$event->set_data_layer( new DataLayer( [ Automation::class => $automation ] ) );
		$event->initialize();

		return $automation;
	}

	public function create_automation( int $automation_id ): Automation {
		if ( isset( $this->automations[ $automation_id ] ) ) {
			return $this->automations[ $automation_id ];
		}

		$this->automations[ $automation_id ] = $this->automation_factory->create( $automation_id );

		return $this->automations[ $automation_id ];
	}

	public function create_runner( Automation $automation ): \WPDesk\ShopMagic\Workflow\Automation\AutomationRunner {
		$validator = clone $this->get_validator();
		$validator
			->add_validator( new FullyConfiguredValidator( $automation ) )
			->add_validator( new FiltersValidator( $automation->get_filters() ) );

		$runner = new AutomationRunner( $automation,
			$validator,
			$this->execution_factory,
			$this->outcome_saver
		);

		$runner->setLogger( $this->logger );

		return $runner;
	}

	public function get_validator(): AutomationValidator {
		return $this->validator ?? new AutomationValidator();
	}

	public function set_validator( AutomationValidator $validator ): void {
		$this->validator = $validator;
	}
}
