<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Automation;

use WPDesk\ShopMagic\Components\Database\Abstraction\PostObjectManager;
use WPDesk\ShopMagic\Recipe\Recipe;
use WPDesk\ShopMagic\Workflow\Automation\ValueObject\ActionValueObject;
use WPDesk\ShopMagic\Workflow\Automation\ValueObject\AutomationValueObject;
use WPDesk\ShopMagic\Workflow\Automation\ValueObject\FilterValueObject;

/**
 * @extends PostObjectManager<AutomationValueObject>
 */
class AutomationObjectManager extends PostObjectManager {

	/** @var \WPDesk\ShopMagic\Components\Validator\AutomationValidator */
	private $validator;

	public function __construct(
		\WPDesk\ShopMagic\Components\Validator\AutomationValidator $validator,
		AutomationValueObjectRepository $repository
	) {
		$this->validator = $validator;
		parent::__construct( $repository );
	}

	/**
	 * Performs soft delete of automation object.
	 * If object is already trashed, deletes it completely.
	 */
	public function do_delete( object $item ): bool {
		if ( $item->get_status() === 'trash' ) {
			return (bool) wp_delete_post( $item->get_id() );
		} else {
			return (bool) wp_trash_post( $item->get_id() );
		}
	}

	public function can_handle( object $item ): bool {
		return $item instanceof AutomationValueObject;
	}

	protected function expected_post_type(): ?string {
		return AutomationPostType::TYPE;
	}

	protected function do_save( object $item ): bool {
		$valid = $this->validator->validate( $item );
		if ( ! $valid ) {
			throw new AutomationNotSaved( 'Validation not passed' );
		}

		if ( $item->get_id() !== null ) {
			wp_update_post( [
				'ID'          => $item->get_id(),
				'post_title'  => $item->get_name(),
				'post_status' => $item->get_status(),
			] );
		} else {
			$id = wp_insert_post(
				[
					'post_title'   => $item->get_name(),
					'post_content' => '',
					'post_type'    => AutomationPostType::TYPE,
					'post_status'  => $item->get_status(),
				]
			);
			$item->set_id( $id );
		}

		$persistence = new AutomationPersistence( $item->get_id() );
		$event       = $item->get_event();
		$persistence->set_event_data( $event->get_settings(), $event->get_name() );
		$persistence->set_actions_data( $this->normalize_actions( $item->get_actions() ) );
		$persistence->set_filters_data( $this->normalize_filters( $item->get_filters() ) );
		$persistence->set( 'lang', $item->get_language() );
		if ( $item->is_from_recipe() ) {
			$persistence->set( Recipe::RECIPE_META_NAME, $item->get_id() );
		}
		return true;
	}

	public function normalize_actions( array $actions ): array {
		return array_map(
			function ( ActionValueObject $action ) {
				return array_merge(
					[
						'_action' => $action->get_name(),
					],
					$action->get_settings()
				);
			},
			array_values( $actions )
		);
	}

	public function normalize_filters( array $filters ): array {
		return array_map(
			function ( array $filters ) {
				return array_map(
					function ( FilterValueObject $filter ) {
						return [
							'data'        => $filter->get_settings(),
							'filter_slug' => $filter->get_id(),
						];
					},
					array_values( $filters )
				);
			},
			array_values( $filters )
		);
	}
}
