<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Automation;

use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use WPDesk\ShopMagic\Exception\ShopMagicException;
use WPDesk\ShopMagic\Workflow\Action\Action;
use WPDesk\ShopMagic\Workflow\ActionExecution\ExecutionCreator\ExecutionCreator;
use WPDesk\ShopMagic\Workflow\Event\DataLayer;
use WPDesk\ShopMagic\Workflow\Outcome\OutcomeSavingState;

/**
 * Prepares, validates and runs automation.
 */
final class AutomationRunner implements Runner, LoggerAwareInterface {
	use LoggerAwareTrait;

	/** @var Automation */
	private $automation;

	/** @var AutomationValidator */
	private $validator;

	/** @var ExecutionCreator */
	private $factory;

	/** @var DataLayer|null */
	private $data_layer;

	/** @var OutcomeSavingState */
	private $outcome_saver;

	public function __construct(
		Automation $automation,
		AutomationValidator $validator,
		ExecutionCreator $executor,
		OutcomeSavingState $outcome_saver
	) {
		$this->automation      = $automation;
		$this->validator       = $validator;
		$this->factory         = $executor;
		$this->outcome_saver   = $outcome_saver;
	}

	public function run( DataLayer $data_layer ): void {
		try {
			$this->data_layer = $data_layer;
			$this->do_run( $data_layer );
		} catch ( ShopMagicException $e ) {
			$this->logger->error( $e->getMessage() );
		} catch ( \Exception $e ) {
			$this->logger->critical( $e->getMessage(), $e->getTrace() );
		} catch ( \Throwable $e ) {
			$this->logger->alert( $e->getMessage(), $e->getTrace() );
			//throw $e;
		}
	}

	/**
	 * @param DataLayer $data_layer
	 *
	 * @return void
	 */
	public function do_run( DataLayer $data_layer ): void {
		$this->setup( $data_layer );

		$this->logger->debug( 'Validating automation with set of validators.' );
		if ( $this->validator->valid() ) {
			$this->logger->debug( 'Validation succeeded. Proceeding to dispatch actions.' );
			$this->execute_actions();
		} else {
			$this->logger->debug( 'Automation validation failed. Cleaning up and shutting down ShopMagic.' );
		}

		$this->cleanup();
	}

	private function setup( DataLayer $data_layer ): void {
		$this->validator->set_provided_data( $data_layer );

		do_action( 'shopmagic/core/automation/setup', $this );
	}

	private function execute_actions(): void {
		$this->logger->debug( sprintf('Executing all actions for automation %d.', $this->automation->get_id() ));
		foreach ( array_values( $this->automation->get_actions() ) as $index => $action ) {
			if ( $this->should_execute_action( $action ) ) {
				$this->logger->debug( sprintf( 'Preparing action `%s` for dispatch.', get_class( $action ) ) );
				$this->delegate_for_execution( $action, $index );
				$this->logger->debug( sprintf( 'Action `%s` dispatched.', get_class( $action ) ) );
			} else {
				$this->logger->info( sprintf( 'Action `%s` skipped.', get_class( $action ) ) );
			}
		}
		$this->logger->debug(
			sprintf(
				'Finished executing all actions for automation %d.',
				$this->automation->get_id()
			)
		);
	}

	private function should_execute_action( Action $action ): bool {
		return apply_filters( 'shopmagic/core/automation/should_execute_action',
			true,
			$this->automation,
			$this->automation->get_event(),
			$action
		);
	}

	private function delegate_for_execution( Action $action, int $index ): void {
		$executor  = $this->factory->create_executor( $this->automation, $this->automation->get_event(), $action );
		$unique_id = $this->outcome_saver->create_outcome( $this->automation, $this->data_layer, $index );
		$this->logger->debug(
			sprintf(
				'Delegating action `%s` to executor `%s`.',
				get_class( $action ),
				get_class( $executor )
			)
		);
		$executor->execute(
			$this->automation,
			$this->automation->get_event(),
			$action,
			$index,
			(string) $unique_id
		);
	}

	private function cleanup(): void {
		do_action( 'shopmagic/core/automation/cleanup', $this );
	}

}
