<?php

declare( strict_types=1 );

namespace WPDesk\ShopMagic\Workflow\Event;

use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use ShopMagicVendor\WPDesk\Forms\FieldProvider;
use ShopMagicVendor\WPDesk\Forms\FieldsDataReceiver;
use WPDesk\ShopMagic\DataSharing\DataProvider;
use WPDesk\ShopMagic\DataSharing\Traits\FieldsDataAsProtectedField;
use WPDesk\ShopMagic\Exception\ReferenceNoLongerAvailableException;
use WPDesk\ShopMagic\Extensions\Elements\GroupableNamedElement;
use WPDesk\ShopMagic\Workflow\Automation\Automation;
use WPDesk\ShopMagic\Workflow\Automation\Runner;


abstract class Event implements
	EventInterface,
	GroupableNamedElement,
	DataProvider,
	FieldProvider,
	FieldsDataReceiver,
	LoggerAwareInterface,
	\JsonSerializable {

	use FieldsDataAsProtectedField;
	use LoggerAwareTrait;

	/** @var DataLayer|null */
	protected $resources;

	/** @var Runner|null */
	private $runner;

	/**
	 * Reverse jsonSerialize. What has been serialized should be possible to unserialize.
	 *
	 * @throws ReferenceNoLongerAvailableException When object reference from json no longer points to a real object.
	 * @see        self::jsonSerialize()
	 * @deprecated 3.0 Use ::denormalize() for better semantics.
	 */
	abstract public function set_from_json( array $serialized_json ): void;

	//abstract public function denormalize( array $data ): void;
	//
	//abstract public function normalize(): array;

	/**
	 * @deprecated 3.0 Event should be normalized explicitly when needed.
	 *             For this purpose use ::normalize()
	 */
	abstract public function jsonSerialize(): array;

	public function get_description(): string {
		return __( 'No description provided for this event.', 'shopmagic-for-woocommerce' );
	}

	/**
	 * Event can be configured with fields.
	 *
	 * @return \ShopMagicVendor\WPDesk\Forms\Field[]
	 */
	public function get_fields(): array {
		return [];
	}

	/**
	 * Each event provides some data through DataLayer, but those have to be registered before we will be able to
	 * access them. Events are filled with Automation::class resource by default during runtime. If Events exposes more
	 * domains to be accessible, overload the method.
	 *
	 * @return string[]
	 * @see DataLayer
	 */
	public function get_provided_data_domains(): array {
		return [ Automation::class ];
	}

	/**
	 * As Events are accessed through Proxy Pattern we have to release non-static resources while cloning an object.
	 *
	 * @return void
	 */
	public function __clone() {
		$this->resources   = null;
		$this->runner      = null;
		$this->fields_data = null;
	}

	final public function trigger_automation(): void {
		$this->logger->debug( sprintf( 'Automation triggered with event %s', static::class ) );
		$this->runner->run( $this->get_provided_data() );
	}

	public function get_provided_data(): DataLayer {
		return $this->resources;
	}

	public function set_runner( Runner $runner ): void {
		$this->runner = $runner;
	}

	public function set_data_layer( DataLayer $resources ): void {
		$this->resources = $resources;
	}

}
